#include "KinectTracker.h"

KinectTracker::KinectTracker()
{
	init();
}

KinectTracker::~KinectTracker()
{
}

bool KinectTracker::init()
{
	// here we initialize our communication with the Kinect through OpenNI
	XnStatus rc = XN_STATUS_OK;

	xn::EnumerationErrors errors;
	// this file includes information about the resolution and frames per second
	rc = m_context.InitFromXmlFile("OpenNiConfig.xml", &errors);

	if (rc == XN_STATUS_NO_NODE_PRESENT)
	{
		XnChar strError[1024];
		errors.ToString(strError, 1024);
		std::cout << strError << std::endl;
	}
	else if (rc != XN_STATUS_OK)
	{
		std::cout << "Open failed: " << xnGetStatusString(rc) << std::endl;
		return false;
	}

	// currently we only create an image generator for the color image
	rc = m_context.FindExistingNode(XN_NODE_TYPE_IMAGE, m_imageGenerator);

	// all generator start generating the images
	rc = m_context.StartGeneratingAll();
	if (rc != XN_STATUS_OK)
	{
		std::cout << "Start generating failed: " << xnGetStatusString(rc) << std::endl;
		return false;
	}

	// meta data containes e.g. information about the resolution
	m_imageGenerator.GetMetaData(m_imageMD);
	
	return true;
}


// return the video image
unsigned char* KinectTracker::getImage()
{
	m_context.WaitAnyUpdateAll();
	m_imageGenerator.GetMetaData(m_imageMD);
	return (unsigned char*)m_imageMD.RGB24Data();
}

// return the depth image
// currently we do not return anything
unsigned short* KinectTracker::getDepthImage()
{
	return 0;
}