// this include is for glut a window manager for OpenGL that creates the window
#include "GL\glut.h"

// this is our tracker
#include "KinectTracker.h"

// variables
// pointer to our own KinectTracker class
KinectTracker*	m_myTracker = 0;
// textures for the Kinect video and depth stream
GLuint				m_videoTexture = 0;
GLuint				m_depthTexture = 0;

// this is the standard function that is called everytime the image is redrawn
void display()
{
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	// you should always set a pointer to 0 when initializing it
	// if you do not assign it to 0 it might point at some arbitrary point
	// if it is set to 0 we can check later whether the pointer is valid
	unsigned char* videoImage = 0;
	unsigned short* depthImage = 0;

	// get the Kinect video image
	videoImage = m_myTracker->getImage();
	depthImage = m_myTracker->getDepthImage();

	glEnable(GL_TEXTURE_2D);

	// if the image is available load the video image into a texture
	// currently this only supports a fixed resolutin
	if(videoImage != 0)
	{
		glBindTexture(GL_TEXTURE_2D, m_videoTexture);
		glTexSubImage2D(GL_TEXTURE_2D, 0, 0, 0, 640, 480, GL_RGB, GL_UNSIGNED_BYTE, videoImage);
	}

	// load the depth image into a texture
	// currently this only supports a fixed resolutin
	// the depth image we receive from OpenNI does only use a small part of the range of available values
	// if we would just upload it to the texture it would only use very low intesities
	// therefor we scale the values while loading it into the texture to use the full range of intensities
	if(depthImage != 0)
	{
		glBindTexture(GL_TEXTURE_2D, m_depthTexture);
		glPixelTransferf(GL_RED_SCALE, 15.0f);
		glPixelTransferf(GL_GREEN_SCALE, 15.0f);
		glPixelTransferf(GL_BLUE_SCALE, 15.0f);
		glTexSubImage2D(GL_TEXTURE_2D, 0, 0, 0, 640, 480, GL_LUMINANCE, GL_UNSIGNED_SHORT, depthImage);
		glPixelTransferf(GL_RED_SCALE, 1.0f);
		glPixelTransferf(GL_GREEN_SCALE, 1.0f);
		glPixelTransferf(GL_BLUE_SCALE, 1.0f);
	}

	// we set up the projection and modelview matrix for the OpenGL drawing
	glMatrixMode(GL_PROJECTION);
	glPushMatrix();
	glLoadIdentity();
	glOrtho(+0.5, -0.5, +0.5, -0.5, 0.001f, 1.0);

	glMatrixMode(GL_MODELVIEW);
	glPushMatrix();
	glLoadIdentity();
	glTranslatef(0.0f, 0.0f, -0.5f);

	// here we do the actual drawing of the texture
	glBindTexture(GL_TEXTURE_2D, m_videoTexture);

	glBegin(GL_QUADS);
		glTexCoord2f(1.0f, 0.0f);
		glVertex3f(-0.5f, -0.5f, 0.0f);
		glTexCoord2f(1.0f, 1.0f);
		glVertex3f(-0.5f, 0.5f, 0.0f);
		glTexCoord2f(0.0f, 1.0f);
		glVertex3f(0.5f, 0.5f, 0.0f);
		glTexCoord2f(0.0f, 0.0f);
		glVertex3f(0.5f, -0.5f, 0.0f);
	glEnd();

	glDisable(GL_TEXTURE_2D);

	glPopMatrix();
	glMatrixMode(GL_PROJECTION);
	glPopMatrix();
	glMatrixMode(GL_MODELVIEW);

	glutSwapBuffers();
}

void reshape(int w, int h)
{
	glViewport(0, 0, w, h);
}

void mouse(int button, int state, int mx, int my)
{
}

void mouseMotion(int x, int y)
{
}

void keyboard(unsigned char key, int x, int y)
{
}

void idle(void)
{
	// we have to call this function so that glut will call display()
	glutPostRedisplay();
}

void init()
{
	// initalize tracker
	m_myTracker = new KinectTracker();

	// set background color when clearing the screen
	glClearColor(0.0f, 0.0f, 0.0f, 1.0f);

	// initialize textures
	glEnable(GL_TEXTURE_2D);

	// initialize a texture to store the kinect video image
	// the video image is a color image with RGB information for each pixel
	// currently this only supports a fixed texture size
	glGenTextures(1, &m_videoTexture);
	glBindTexture(GL_TEXTURE_2D, m_videoTexture);
	glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB, 640, 480, 0, GL_RGB, GL_UNSIGNED_BYTE, 0);
	// we have to set some parameters for OpenGL to do correct texturing
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);

	// initialize a texture to store the kinect depth image
	// the depth image contains only the depth information for each pixel
	// currently this only supports a fixed texture size
	glGenTextures(1, &m_depthTexture);
	glBindTexture(GL_TEXTURE_2D, m_depthTexture);
	glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB, 640, 480, 0, GL_LUMINANCE, GL_UNSIGNED_SHORT, 0);
	glDisable(GL_TEXTURE_2D);
	// we have to set some parameters for OpenGL to do correct texturing
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
}

void main(int argc, char** argv)
{
	// initialize glut
	// glut is a toolkit that creates an OpenGL window, sets up an OpenGL context and manages things like keyboard or mouse input
	// an alternative is e.g. Qt which allows to create more sophisticated user interfaces
	glutInit(&argc, argv);
	glutInitDisplayMode(GLUT_RGB | GLUT_DOUBLE);
	glutInitWindowSize(640, 480);
	glutCreateWindow("Kinect Lab Course");

	init();

	// define callback functions
	glutDisplayFunc(display);
	glutIdleFunc(idle);
	glutReshapeFunc(reshape);
	glutMouseFunc(mouse);
	glutMotionFunc(mouseMotion);
	glutKeyboardFunc(keyboard);

	// run main loop
	glutMainLoop();
}